static char azn_ent_svc_ClientProtocol_c_sourceCodeID[]="$Id: 1 03 1.1 src/azn_ent_svc_ClientProtocol.c, if.am.sapwas, ifamsapwas3.0, 05 11/24/05 19:11:55 $";

/*
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplicaion or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * FILENAME
 *      azn_ent_svc_ClientProtocol.c
 *
 * DESCRIPTION
 *      Sample entitlements service for integration with 
 *      SAP Web Application Server Java when using protocol switching.
 */

#include <stdlib.h>
#include <stdio.h>

#include <ctype.h>
#include <ogauthzn.h>
#include <azn_svc_protos.h>
#include <aznutils.h>


// this version
#define ENT_SVC_VER "TAM entitlements service - ClientProtocol v1.0"

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

#ifdef __cplusplus
extern "C" {
#endif


void attrlist_dump(azn_attrlist_h_t);
void check_status(char*, unsigned);

/*
 * Dump_cred will be set to true if we should print out the contents of
 * the credential before asking for new attributes.
 */
int _dump_cred = FALSE;
int	_trace = FALSE;


/******************************************************************************
 * Interface function definitions.                                            *
 ******************************************************************************/

/**
 * FUNCTION NAME
 *      azn_svc_initialize
 *
 * DESCRIPTION
 *      init the credattrs entitlements service
 *
 * ARGUMENTS
 *      [in] argc       The count of arguments to the service.
 *      [in] argv       The array of argument strings.
 *      [in] svcInit    List of initialization attributes for the service.
 *      [out] svcInfo   attr list ptr for attributes returned by the service.
 *
 * RETURN VALUE
 *      AZN_S_COMPLETE on success, error code on failure
 */
AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_initialize(const int                 argc,       /* in */
                   const char                **argv,     /* in */
                   const azn_attrlist_h_t    svcInit,    /* in */
                   azn_attrlist_h_t          *svcInfo    /* out */
                  )
{
    azn_status_t status = AZN_S_COMPLETE;
    azn_string_t svcid = NULL;
    azn_string_t azn_code_set = NULL;
    azn_string_t conf_file = NULL;
    azn_string_t domain = NULL;
    azn_boolean_t       freeAttrlist = FALSE;
    int i=0;

    if (svcInfo == NULL)
    {
        return(azn_util_errcode(AZN_S_SVC_ENT_INVALID_SVCINFO_HDL, 0));
    }

    if ((argc < 0) || (argc == 0 && argv != NULL))
    {
        return(azn_util_errcode(AZN_S_SVC_ENT_INVALID_ARG_COUNT, 0));
    }

    if ((argc > 0) && (argv == NULL))
    {
        return(azn_util_errcode(AZN_S_SVC_ENT_INVALID_ARG_COUNT, 0));
    }

    /* print the arguments */
	printf("\n%s\nazn_svc_initialize() Parameters:\n", ENT_SVC_VER);
    for (i=0; i < argc; i++)
    {
        printf("param[%i] = %s\n", i, argv[i]);
		if (strcmp(argv[i], "-dump_cred") == 0)
        {
            printf("...displaying input credentials\n");
            _dump_cred = TRUE;
        }
		if (strcmp(argv[i], "-trace") == 0)
        {
            printf("...displaying trace information\n");
            _trace = TRUE;
        }
    }

    // get my service id - must have
    status = azn_attrlist_get_entry_string_value(svcInit,
                                                 azn_svc_init_my_service_id,
                                                 AZN_C_INITIATOR_INDEX,
                                                 &svcid);
    if (status == AZN_S_COMPLETE || svcid != NULL)
    {
        if (_trace)
			printf("Entitilement Service ClientProtocol ID: %s\n", svcid);
        azn_release_string(&svcid);
    }
    else
    {
        return(azn_util_errcode(status, 0));
    }

    /* return the service version to the dispatcher */
    if (*svcInfo == AZN_C_INVALID_HANDLE)
    {
        azn_attrlist_create(svcInfo);
        freeAttrlist = TRUE;
    }
    // add this version info to svcInfo
    status = azn_attrlist_add_entry(*svcInfo,
                                    (const azn_string_t)azn_svc_version,
                                    (const azn_string_t)ENT_SVC_VER);
    if (status != AZN_S_COMPLETE)
    {
        if (freeAttrlist)
        {
            azn_attrlist_delete(svcInfo);
        }
    }

	if (_trace)
		printf( "azn_svc_initialize() complete\n", ENT_SVC_VER );

    return(azn_util_errcode(AZN_S_COMPLETE, 0));
}


/**
 * FUNCTION NAME
 *      azn_svc_shutdown
 *
 * DESCRIPTION
 *      shutdown the ext_attr entitlements service.
 *      The initialization parameters are passed in
 *      again on shutdown but are ignored. No version info
 *      is returned.
 *
 * ARGUMENTS
 *      [in] argc       The count of arguments to the service.
 *      [in] argv       The array of argument strings.
 *      [in] svcInit    List of initialization attributes for the service.
 *      [out] svcInfo   attr list ptr for attributes returned by the service.
 *
 * RETURN VALUE
 *      AZN_S_COMPLETE on success, error code on failure
 */

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_shutdown(const int                 argc,       // in
                 const char                **argv,     // in
                 const azn_attrlist_h_t    svcInit,    // in
                 azn_attrlist_h_t          *svcInfo    // out
                )
{
	if (_trace)
		printf( "%s shutdown complete\n", ENT_SVC_VER );
    return(azn_util_errcode(AZN_S_COMPLETE, 0));
}


/**
 * FUNCTION NAME
 *      azn_svc_entitlement_get_entitlements
 *
 * DESCRIPTION
 *      Returns extended attribute information associated with
 *      the cred passed in
 *
 * ARGUMENTS
 *      [in]    creds           The credentials of the caller
 *      [in]    svc_id          The id of the entitlements service
 *      [in]    app_context     attribute list containing information
 *                              on which extended attributes to get
 *      [out]   entitlements    attribute list containing the extended
 *                              attributes
 *
 * RETURN VALUE
 *      AZN_S_COMPLETE on success, error code on failure
 *
 *
 */

AZN_DECLSPEC
azn_status_t
AZN_CALLTYPE
azn_svc_entitlement_get_entitlements(
			    const azn_creds_h_t       creds,         /* in */
			    const azn_string_t        svc_id,        /* in */
			    const azn_attrlist_h_t    app_context,   /* in */
			    azn_attrlist_h_t          *entitlements) /* out */
{

    azn_status_t status = AZN_S_COMPLETE;
    unsigned num = 0;
    unsigned numval = 0;
    int      i = 0;
    int      j = 0;
    azn_attrlist_h_t cred_attrs = AZN_C_INVALID_HANDLE;
    azn_boolean_t free_enthdl = FALSE;

    if (_trace)
		printf("\n%s\nazn_svc_entitlement_get_entitlements()\n", ENT_SVC_VER);

    // check that we dont have an invalid handle
    if (entitlements == NULL)
    {
        return(azn_util_errcode(AZN_S_INVALID_ENTITLEMENTS_HDL, 0));
    }

    if (_trace)
		printf("Service ID: %s\n", svc_id);

    if (*entitlements == AZN_C_INVALID_HANDLE)
    {
        status = azn_attrlist_create(entitlements);
        if (status != AZN_S_COMPLETE)
        {
            return(status);
        }
        free_enthdl = TRUE;

    }

	if (_dump_cred)
    {
        /* Display the attrlist for the principal */
        status = azn_creds_get_attrlist_for_subject(creds,
                                                    AZN_C_INITIATOR_INDEX, &cred_attrs);
        check_status((char*)"azn_creds_get_attrlist_for_subject", status);
        if (status == AZN_S_COMPLETE)
        {
            if (_trace)
				printf("Current credential attributes:\n");
            attrlist_dump(cred_attrs);
            azn_attrlist_delete(&cred_attrs);
        }
    }

    /* Add the ClientProtocol information to the credential */
	status = azn_attrlist_add_entry(*entitlements, (const azn_string_t)"tagvalue_ClientProtocol", (const azn_string_t)"https");
	check_status((char *)"azn_attrlist_add_entry", status);
	if (_trace)
		printf("Added tagvalue_ClientProtocol: https\n");

    return(status);
}

/*
 * check status and print a message.
 */
void
check_status(
            char *info,
            unsigned status
            )
{
    azn_string_t errstr = NULL;
    unsigned majerr, minerr;
    azn_status_t rc;

    if (status == AZN_S_COMPLETE)
        return;

    majerr = azn_error_major(status);
    minerr = azn_error_minor(status);

    if (_trace)
		printf("\n%s: ", info);

    rc = azn_error_get_string(status, &errstr);
    if (rc == AZN_S_COMPLETE && errstr)
    {
        if (_trace)
			printf("%s (0x%08x/0x%08x)\n\n", errstr, majerr, minerr);
        azn_release_string(&errstr);
    }
    else
	{
        if (_trace)
			printf("API failure (0x%08x/0x%08x)\n\n", majerr, minerr);
	}
}

void attrlist_dump(azn_attrlist_h_t perminfo)
{
    azn_string_t* attr_names;
    int i ,j;
    unsigned int count;
    azn_status_t    aznStatus;
    azn_buffer_t buffer_value;
    azn_string_t str_value;
    azn_ulong_t                 ulong_value;

    aznStatus = azn_attrlist_get_names(perminfo, &attr_names);
    check_status((char *)"azn_attrlist_get_names", aznStatus);
    if (attr_names)
    {
        for (i = 0; attr_names[i] != NULL ;i++)
        {
            aznStatus = azn_attrlist_name_get_num(perminfo,
                                                  attr_names[i],  &count);
            check_status((char *)"azn_attrlist_name_get_num", aznStatus);

            printf("\n-----------------------------\n");
            printf("Attr Name = %s\n" , attr_names[i]);
            printf("Values are : \n");
            for (j=0;j<count;j++)
            {
                aznStatus = azn_attrlist_get_entry_string_value(perminfo,attr_names[i],j,&str_value);
                if (aznStatus == AZN_S_COMPLETE)
                {
                    printf("%s\n", str_value);
                    azn_release_string(&str_value);
                }
                else
                {
                    aznStatus = azn_attrlist_get_entry_buffer_value(perminfo,attr_names[i],j,&buffer_value);
                    if (aznStatus == AZN_S_COMPLETE)
                    {
                        printf("0x%08x\n" , buffer_value);
                        azn_release_buffer(&buffer_value);
                    }
                    else
                    {
                        aznStatus = azn_attrlist_get_entry_ulong_value(perminfo,attr_names[i],j,&ulong_value);
                        if (aznStatus == AZN_S_COMPLETE)
                            printf("%d\n" , ulong_value);
                    }

                }
            }
        }
    }
}


#ifdef __cplusplus
}
#endif

